@@ -14,9 +14,13 @@ from mergedeep import merge  # type: ignore
 from sticker_convert.definitions import CONFIG_DIR, DEFAULT_DIR
 from sticker_convert.job import Job
 from sticker_convert.job_option import CompOption, CredOption, InputOption, OutputOption
+from sticker_convert.utils.auth.get_discord_auth import GetDiscordAuth
 from sticker_convert.utils.auth.get_kakao_auth import GetKakaoAuth
+from sticker_convert.utils.auth.get_kakao_desktop_auth import GetKakaoDesktopAuth
 from sticker_convert.utils.auth.get_line_auth import GetLineAuth
 from sticker_convert.utils.auth.get_signal_auth import GetSignalAuth
+from sticker_convert.utils.auth.get_viber_auth import GetViberAuth
+from sticker_convert.utils.auth.telethon_setup import TelethonSetup
 from sticker_convert.utils.callback import Callback
 from sticker_convert.utils.files.json_manager import JsonManager
 from sticker_convert.utils.url_detect import UrlDetect
@@ -67,26 +71,26 @@ class CLI:
 
         parser_input = parser.add_argument_group("Input options")
         for k, v_str in self.help["input"].items():
-            parser_input.add_argument(f'--{k.replace("_", "-")}', dest=k, help=v_str)
+            parser_input.add_argument(f"--{k.replace('_', '-')}", dest=k, help=v_str)
         parser_input_src = parser_input.add_mutually_exclusive_group()
         for k, v_dict in self.input_presets.items():
             if k == "local":
                 continue
             parser_input_src.add_argument(
-                f'--download-{k.replace("_", "-")}',
+                f"--download-{k.replace('_', '-')}",
                 dest=f"download_{k}",
-                help=f'{v_dict["help"]}\n({v_dict["example"]})',
+                help=f"{v_dict['help']}\n({v_dict['example']})",
             )
 
         parser_output = parser.add_argument_group("Output options")
         for k, v_str in self.help["output"].items():
-            parser_output.add_argument(f'--{k.replace("_", "-")}', dest=k, help=v_str)
+            parser_output.add_argument(f"--{k.replace('_', '-')}", dest=k, help=v_str)
         parser_output_dst = parser_output.add_mutually_exclusive_group()
         for k, v_dict in self.output_presets.items():
             if k == "local":
                 continue
             parser_output_dst.add_argument(
-                f'--export-{k.replace("_", "-")}',
+                f"--export-{k.replace('_', '-')}",
                 dest=f"export_{k}",
                 action="store_true",
                 help=v_dict["help"],
@@ -150,7 +154,7 @@ class CLI:
             else:
                 continue
             parser_comp.add_argument(
-                f'--{k.replace("_", "-")}',
+                f"--{k.replace('_', '-')}",
                 **keyword_args,
                 dest=k,
                 help=v,
@@ -163,13 +167,21 @@ class CLI:
         )
 
         parser_cred = parser.add_argument_group("Credentials options")
-        flags_cred_bool = ("signal_get_auth", "kakao_get_auth", "line_get_auth")
+        flags_cred_bool = (
+            "signal_get_auth",
+            "telethon_setup",
+            "kakao_get_auth",
+            "kakao_get_auth_desktop",
+            "line_get_auth",
+            "discord_get_auth",
+            "save_cred",
+        )
         for k, v in self.help["cred"].items():
             keyword_args = {}
             if k in flags_cred_bool:
                 keyword_args = {"action": "store_true"}
             parser_cred.add_argument(
-                f'--{k.replace("_", "-")}',
+                f"--{k.replace('_', '-')}",
                 **keyword_args,
                 dest=k,
                 help=v,
@@ -217,8 +229,11 @@ class CLI:
             "signal": args.download_signal,
             "line": args.download_line,
             "telegram": args.download_telegram,
+            "telegram_telethon": args.download_telegram_telethon,
             "kakao": args.download_kakao,
             "viber": args.download_viber,
+            "discord": args.download_discord,
+            "discord_emoji": args.download_discord_emoji,
         }
 
         download_option = "local"
@@ -255,6 +270,14 @@ class CLI:
             export_option = "signal"
         elif args.export_telegram:
             export_option = "telegram"
+        elif args.export_telegram_emoji:
+            export_option = "telegram_emoji"
+        elif args.export_telegram_telethon:
+            export_option = "telegram_telethon"
+        elif args.export_telegram_emoji_telethon:
+            export_option = "telegram_emoji_telethon"
+        elif args.export_viber:
+            export_option = "viber"
         elif args.export_imessage:
             export_option = "imessage"
         else:
@@ -280,6 +303,10 @@ class CLI:
                         args.export_whatsapp,
                         args.export_signal,
                         args.export_telegram,
+                        args.export_telegram_emoji,
+                        args.export_telegram_telethon,
+                        args.export_telegram_emoji_telethon,
+                        args.export_viber,
                         args.export_imessage,
                     )
                 )
@@ -291,8 +318,12 @@ class CLI:
                 preset = "whatsapp"
             elif args.export_signal:
                 preset = "signal"
-            elif args.export_telegram:
+            elif args.export_telegram or args.export_telegram_telethon:
                 preset = "telegram"
+            elif args.export_telegram_emoji or args.export_telegram_emoji_telethon:
+                preset = "telegram_emoji"
+            elif args.export_viber:
+                preset = "viber"
             elif args.export_imessage:
                 preset = "imessage_small"
         elif args.preset == "auto":
@@ -305,6 +336,12 @@ class CLI:
                 self.cb.msg(
                     "Auto compression option set to no_compress (Reason: Export to local directory only)"
                 )
+            elif "telegram_emoji" in output_option:
+                preset = "telegram_emoji"
+                self.cb.msg(f"Auto compression option set to {preset}")
+            elif "telegram" in output_option:
+                preset = "telegram"
+                self.cb.msg(f"Auto compression option set to {preset}")
             elif output_option == "imessage":
                 preset = "imessage_small"
                 self.cb.msg(f"Auto compression option set to {preset}")
@@ -434,6 +471,8 @@ class CLI:
             telegram_userid=args.telegram_userid
             if args.telegram_userid
             else creds.get("telegram", {}).get("userid"),
+            telethon_api_id=creds.get("telethon", {}).get("api_id"),
+            telethon_api_hash=creds.get("telethon", {}).get("api_hash"),
             kakao_auth_token=args.kakao_auth_token
             if args.kakao_auth_token
             else creds.get("kakao", {}).get("auth_token"),
@@ -452,6 +491,12 @@ class CLI:
             line_cookies=args.line_cookies
             if args.line_cookies
             else creds.get("line", {}).get("cookies"),
+            viber_auth=args.viber_auth
+            if args.viber_auth
+            else creds.get("viber", {}).get("auth"),
+            discord_token=args.discord_token
+            if args.discord_token
+            else creds.get("discord", {}).get("token"),
         )
 
         if args.kakao_get_auth:
@@ -468,24 +513,41 @@ class CLI:
 
                 self.cb.msg(f"Got auth_token successfully: {auth_token}")
 
-        if args.signal_get_auth:
-            get_signal_auth = GetSignalAuth()
-
-            signal_bin_path = None
-            signal_user_data_dir = None
-            if args.signal_data_dir:
-                signal_bin_path = "(User specified)"
-                signal_user_data_dir = args.signal_data_dir
-
-            uuid, password, msg = get_signal_auth.get_cred(
-                signal_bin_path, signal_user_data_dir
+        if args.kakao_get_auth_desktop:
+            get_kakao_desktop_auth = GetKakaoDesktopAuth(
+                cb_ask_str=self.cb.ask_str,
             )
+            kakao_bin_path = None
+            if args.kakao_bin_path:
+                kakao_bin_path = args.kakao_bin_path
+            auth_token, msg = get_kakao_desktop_auth.get_cred(kakao_bin_path)
 
+            if auth_token:
+                opt_cred.kakao_auth_token = auth_token
+
+            self.cb.msg(msg)
+
+        if args.signal_get_auth:
+            m = GetSignalAuth(cb_msg=self.cb.msg, cb_ask_str=self.cb.ask_str)
+
+            uuid, password = m.get_cred()
             if uuid and password:
                 opt_cred.signal_uuid = uuid
                 opt_cred.signal_password = password
 
-            self.cb.msg(msg)
+                self.cb.msg(f"Got uuid and password successfully: {uuid}, {password}")
+
+            self.cb.msg("Failed to get uuid and password")
+
+        if args.telethon_setup:
+            telethon_setup = TelethonSetup(opt_cred, self.cb.ask_str)
+            success, _, telethon_api_id, telethon_api_hash = telethon_setup.start()
+
+            if success:
+                opt_cred.telethon_api_id = telethon_api_id
+                opt_cred.telethon_api_hash = telethon_api_hash
+
+                self.cb.msg("Telethon setup successful")
 
         if args.line_get_auth:
             get_line_auth = GetLineAuth()
@@ -501,6 +563,29 @@ class CLI:
                     "Failed to get Line cookies. Have you logged in the web browser?"
                 )
 
+        if args.viber_get_auth:
+            get_viber_auth = GetViberAuth(self.cb.ask_str)
+
+            viber_bin_path = None
+            if args.viber_bin_path:
+                viber_bin_path = args.viber_bin_path
+
+            viber_auth, msg = get_viber_auth.get_cred(viber_bin_path)
+
+            if viber_auth:
+                opt_cred.viber_auth = viber_auth
+
+            self.cb.msg(msg)
+
+        if args.discord_get_auth:
+            get_discord_auth = GetDiscordAuth(self.cb.msg)
+            discord_token, msg = get_discord_auth.get_cred()
+
+            if discord_token:
+                opt_cred.discord_token = discord_token
+
+            self.cb.msg(msg)
+
         if args.save_cred:
             creds_path = CONFIG_DIR / "creds.json"
             JsonManager.save_json(creds_path, opt_cred.to_dict())
