@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+import locale
 import os
 import platform
 import signal
@@ -9,7 +10,7 @@ from math import ceil
 from multiprocessing import Event, cpu_count
 from pathlib import Path
 from threading import Thread
-from typing import Any, Callable, Dict, Optional, Union
+from typing import Any, Callable, Dict, Optional, Union, cast
 from urllib.parse import urlparse
 
 from mergedeep import merge  # type: ignore
@@ -62,6 +63,7 @@ class GUI(Window):
         self.apply_creds()
         self.init_frames()
         self.pack_frames()
+        self.author_info()
         self.warn_tkinter_bug()
         GUIUtils.finalize_window(self)
 
@@ -150,15 +152,20 @@ class GUI(Window):
         # Credentials
         self.signal_uuid_var = StringVar(self)
         self.signal_password_var = StringVar(self)
-        self.signal_data_dir_var = StringVar(self)
         self.telegram_token_var = StringVar(self)
         self.telegram_userid_var = StringVar(self)
+        self.telethon_api_id_var = IntVar(self)
+        self.telethon_api_hash_var = StringVar(self)
         self.kakao_auth_token_var = StringVar(self)
         self.kakao_username_var = StringVar(self)
         self.kakao_password_var = StringVar(self)
         self.kakao_country_code_var = StringVar(self)
         self.kakao_phone_number_var = StringVar(self)
+        self.kakao_bin_path_var = StringVar(self)
         self.line_cookies_var = StringVar(self)
+        self.viber_auth_var = StringVar(self)
+        self.viber_bin_path_var = StringVar(self)
+        self.discord_token_var = StringVar(self)
 
         # Config
         self.settings_save_cred_var = BooleanVar()
@@ -203,6 +210,36 @@ class GUI(Window):
             column=0, row=4, columnspan=2, sticky="news", padx=5, pady=5
         )
 
+    def is_cn(self) -> bool:
+        cn_locs = ("zh_chs", "zh_cn", "chinese")
+        loc = locale.getlocale()[0]
+        winloc = ""
+        if platform.system() == "Windows":
+            import ctypes
+
+            windll = ctypes.windll.kernel32  # type: ignore
+            winloc_id = cast(int, windll.GetUserDefaultUILanguage())  # type: ignore
+            winloc = cast(str, locale.windows_locale[winloc_id])  # type: ignore
+
+        if loc is not None and (
+            loc.lower().startswith(cn_locs) or winloc.lower().startswith(cn_locs)
+        ):
+            return True
+        return False
+
+    def author_info(self) -> None:
+        if self.is_cn():
+            msg = "sticker-convert是laggykiller创作的免费开源项目，严禁翻售\n"
+            msg += "https://github.com/laggykiller/sticker-convert\n"
+            msg += "尊重知识产权，做个文明人，请不要转售贴图！\n"
+        else:
+            msg = "sticker-convert is Free and Opensource software by laggykiller\n"
+            msg += "https://github.com/laggykiller/sticker-convert\n"
+            msg += "Please use the stickers with your friends only.\n"
+            msg += "It is illegal and immoral to sell stickers downloaded from this program.\n"
+
+        self.cb_msg(msg)
+
     def warn_tkinter_bug(self) -> None:
         if (
             platform.system() == "Darwin"
@@ -212,13 +249,9 @@ class GUI(Window):
             and sys.version_info[2] <= 6
         ):
             msg = "NOTICE: If buttons are not responsive, try to press "
-            msg += "on title bar or move mouse cursor away from window for a while."
-            self.cb_msg(msg)
-            msg = (
-                "(This is due to a bug in tkinter specific to macOS 14 python <=3.11.6)"
-            )
-            self.cb_msg(msg)
-            msg = "(https://github.com/python/cpython/issues/110218)"
+            msg += "on title bar or move mouse cursor away from window for a while.\n"
+            msg += "(This is a bug in tkinter specific to macOS 14 python <=3.11.6)\n"
+            msg += "(https://github.com/python/cpython/issues/110218)\n"
             self.cb_msg(msg)
 
     def load_jsons(self) -> None:
@@ -375,6 +408,10 @@ class GUI(Window):
         self.signal_password_var.set(self.creds.get("signal", {}).get("password", ""))
         self.telegram_token_var.set(self.creds.get("telegram", {}).get("token", ""))
         self.telegram_userid_var.set(self.creds.get("telegram", {}).get("userid", ""))
+        self.telethon_api_id_var.set(self.creds.get("telethon", {}).get("api_id", 0))
+        self.telethon_api_hash_var.set(
+            self.creds.get("telethon", {}).get("api_hash", "")
+        )
         self.kakao_auth_token_var.set(self.creds.get("kakao", {}).get("auth_token", ""))
         self.kakao_username_var.set(self.creds.get("kakao", {}).get("username", ""))
         self.kakao_password_var.set(self.creds.get("kakao", {}).get("password", ""))
@@ -385,6 +422,8 @@ class GUI(Window):
             self.creds.get("kakao", {}).get("phone_number", "")
         )
         self.line_cookies_var.set(self.creds.get("line", {}).get("cookies", ""))
+        self.viber_auth_var.set(self.creds.get("viber", {}).get("auth", ""))
+        self.discord_token_var.set(self.creds.get("discord", {}).get("token", ""))
 
     def get_input_name(self) -> str:
         return [
@@ -418,6 +457,10 @@ class GUI(Window):
         selection = self.comp_preset_var.get()
         if selection == "auto":
             output_option = self.get_output_name()
+            if "telegram_emoji" in output_option:
+                return "telegram_emoji"
+            if "telegram" in output_option:
+                return "telegram"
             if output_option == "imessage":
                 return "imessage_small"
             if output_option == "local":
@@ -539,12 +582,16 @@ class GUI(Window):
             signal_password=self.signal_password_var.get(),
             telegram_token=self.telegram_token_var.get(),
             telegram_userid=self.telegram_userid_var.get(),
+            telethon_api_id=self.telethon_api_id_var.get(),
+            telethon_api_hash=self.telethon_api_hash_var.get(),
             kakao_auth_token=self.kakao_auth_token_var.get(),
             kakao_username=self.kakao_username_var.get(),
             kakao_password=self.kakao_password_var.get(),
             kakao_country_code=self.kakao_country_code_var.get(),
             kakao_phone_number=self.kakao_phone_number_var.get(),
             line_cookies=self.line_cookies_var.get(),
+            viber_auth=self.viber_auth_var.get(),
+            discord_token=self.discord_token_var.get(),
         )
 
     def start_process(self) -> None:
@@ -587,7 +634,7 @@ class GUI(Window):
         initialvalue: Optional[str] = None,
         cli_show_initialvalue: bool = True,
         parent: Optional[object] = None,
-    ) -> Any:
+    ) -> str:
         self.action = partial(
             Querybox.get_string,  # type: ignore
             question,
@@ -599,7 +646,12 @@ class GUI(Window):
         self.response_event.wait()
         self.response_event.clear()
 
-        return self.response
+        if self.response is None:
+            return ""
+        elif isinstance(self.response, str):
+            return self.response
+        else:
+            raise RuntimeError(f"Invalid response in cb_ask_str: {self.response}")
 
     def cb_ask_bool(
         self, question: str, parent: Union[Window, Toplevel, None] = None
@@ -654,9 +706,9 @@ class GUI(Window):
             set_progress_mode, steps, update_bar, *args, **kwargs
         )
 
-    def highlight_fields(self) -> bool:
+    def highlight_fields(self):
         if not self.init_done:
-            return True
+            return
 
         input_option = self.get_input_name()
         input_option_display = self.get_input_display_name()
@@ -698,8 +750,13 @@ class GUI(Window):
             if not url:
                 self.input_frame.address_entry.config(bootstyle="warning")  # type: ignore
 
-            elif download_option != input_option and not (
-                input_option in ("kakao", "line") and url.isnumeric()
+            elif (
+                download_option is None
+                or input_option.startswith(download_option) is False
+                and not (
+                    input_option in ("kakao", "line", "discord", "discord_emoji")
+                    and url.isnumeric()
+                )
             ):
                 self.input_frame.address_entry.config(bootstyle="danger")  # type: ignore
                 self.input_frame.address_tip.config(
@@ -771,6 +828,11 @@ class GUI(Window):
         else:
             self.cred_frame.telegram_userid_entry.config(bootstyle="default")  # type: ignore
 
+        if output_option == "viber" and not self.viber_auth_var.get():
+            self.cred_frame.viber_auth_entry.config(bootstyle="warning")  # type: ignore
+        else:
+            self.cred_frame.viber_auth_entry.config(bootstyle="default")  # type: ignore
+
         if (
             urlparse(url).netloc == "e.kakao.com"
             and not self.kakao_auth_token_var.get()
@@ -779,6 +841,11 @@ class GUI(Window):
         else:
             self.cred_frame.kakao_auth_token_entry.config(bootstyle="default")  # type: ignore
 
+        if input_option.startswith("discord") and not self.discord_token_var.get():
+            self.cred_frame.discord_token_entry.config(bootstyle="warning")  # type: ignore
+        else:
+            self.cred_frame.discord_token_entry.config(bootstyle="default")  # type: ignore
+
         # Check for Input and Compression mismatch
         if (
             not self.no_compress_var.get()
@@ -791,5 +858,3 @@ class GUI(Window):
         else:
             self.comp_frame.comp_preset_opt.config(bootstyle="secondary")  # type: ignore
             self.output_frame.output_option_opt.config(bootstyle="secondary")  # type: ignore
-
-        return True
