@@ -12,6 +12,7 @@ from typing import Any, Callable, Dict, List, Optional, Tuple
 from urllib.parse import urlparse
 
 from sticker_convert.converter import StickerConvert
+from sticker_convert.downloaders.download_discord import DownloadDiscord
 from sticker_convert.downloaders.download_kakao import DownloadKakao
 from sticker_convert.downloaders.download_line import DownloadLine
 from sticker_convert.downloaders.download_signal import DownloadSignal
@@ -21,6 +22,7 @@ from sticker_convert.job_option import CompOption, CredOption, InputOption, Outp
 from sticker_convert.uploaders.compress_wastickers import CompressWastickers
 from sticker_convert.uploaders.upload_signal import UploadSignal
 from sticker_convert.uploaders.upload_telegram import UploadTelegram
+from sticker_convert.uploaders.upload_viber import UploadViber
 from sticker_convert.uploaders.xcode_imessage import XcodeImessage
 from sticker_convert.utils.callback import CallbackReturn, CbQueueType, ResultsListType, WorkQueueType
 from sticker_convert.utils.files.json_resources_loader import OUTPUT_JSON
@@ -170,6 +172,10 @@ class Executor:
         try:
             for process in self.processes:
                 process.join()
+                if process.exitcode != 0 and self.is_cancel_job.value == 0:
+                    self.cb_msg(
+                        f"Warning: A process exited with error (code {process.exitcode})"
+                    )
         except KeyboardInterrupt:
             pass
 
@@ -214,7 +220,6 @@ class Job:
         self.cb_ask_bool = cb_ask_bool
         self.cb_ask_str = cb_ask_str
 
-        self.compress_fails: List[str] = []
         self.out_urls: List[str] = []
 
         self.executor = Executor(
@@ -234,19 +239,21 @@ class Job:
 
         self.executor.cb("msg", kwargs={"cls": True})
 
-        tasks = (
+        tasks: Tuple[Callable[..., Tuple[bool, Optional[str]]], ...] = (
             self.verify_input,
             self.cleanup,
             self.download,
             self.compress,
             self.export,
-            self.report,
         )
 
         code = 0
+        summaries: List[str] = []
         for task in tasks:
             self.executor.cb("bar", kwargs={"set_progress_mode": "indeterminate"})
-            success = task()
+            success, summary = task()
+            if summary is not None:
+                summaries.append(summary)
 
             if self.executor.is_cancel_job.value == 1:  # type: ignore
                 code = 2
@@ -256,6 +263,22 @@ class Job:
                 self.executor.cb("An error occured during this run.")
                 break
 
+        msg = "##########\n"
+        msg += "Summary:\n"
+        msg += "##########\n"
+
+        msg += "\n"
+        msg += "\n".join(summaries)
+        msg += "\n"
+
+        if self.out_urls:
+            msg += "Export results:\n"
+            msg += "\n".join(self.out_urls)
+        else:
+            msg += "Export result: None"
+
+        self.executor.cb(msg)
+
         self.executor.cleanup()
 
         return code
@@ -263,7 +286,7 @@ class Job:
     def cancel(self, *_: Any, **_kwargs: Any) -> None:
         self.executor.kill_workers()
 
-    def verify_input(self) -> bool:
+    def verify_input(self) -> Tuple[bool, None]:
         info_msg = ""
         error_msg = ""
 
@@ -294,6 +317,12 @@ class Job:
             )
             error_msg += save_to_local_tip
 
+        if (
+            self.opt_input.option.startswith("discord")
+            and not self.opt_cred.discord_token
+        ):
+            error_msg += "[X] Downloading from Discord requires token.\n"
+
         if self.opt_output.option == "telegram" and not self.opt_cred.telegram_userid:
             error_msg += "[X] Uploading to telegram requires user_id \n"
             error_msg += "    (From real account, not bot account).\n"
@@ -305,6 +334,10 @@ class Job:
             error_msg += "[X] Uploading to signal requires uuid and password.\n"
             error_msg += save_to_local_tip
 
+        if self.opt_output.option == "viber" and not self.opt_cred.viber_auth:
+            error_msg += "[X] Uploading to Viber requires auth data.\n"
+            error_msg += save_to_local_tip
+
         output_presets = OUTPUT_JSON
 
         input_option = self.opt_input.option
@@ -317,7 +350,7 @@ class Job:
                 if not MetadataHandler.check_metadata_provided(
                     self.opt_input.dir, input_option, metadata
                 ):
-                    error_msg += f'[X] {output_presets[output_option]["full_name"]} requires {metadata}\n'
+                    error_msg += f"[X] {output_presets[output_option]['full_name']} requires {metadata}\n"
                     if self.opt_input.option == "local":
                         error_msg += f"    {metadata} was not supplied and {metadata}.txt is absent\n"
                     else:
@@ -327,7 +360,7 @@ class Job:
                     )
                     error_msg += f"    Create {metadata}.txt with the {metadata} name\n"
                 else:
-                    info_msg += f'[!] {output_presets[output_option]["full_name"]} requires {metadata}\n'
+                    info_msg += f"[!] {output_presets[output_option]['full_name']} requires {metadata}\n"
                     if self.opt_input.option == "local":
                         info_msg += f"    {metadata} was not supplied but {metadata}.txt is present\n"
                         info_msg += f"    Using {metadata} name in {metadata}.txt\n"
@@ -340,7 +373,7 @@ class Job:
 
         if error_msg != "":
             self.executor.cb(error_msg)
-            return False
+            return False, None
 
         # Check if preset not equal to export option
         # Only warn if the compression option is available in export preset
@@ -350,7 +383,10 @@ class Job:
             not self.opt_comp.no_compress
             and self.opt_output.option != "local"
             and self.opt_comp.preset != "custom"
-            and self.opt_output.option not in self.opt_comp.preset
+            and (
+                self.opt_output.option not in self.opt_comp.preset
+                and self.opt_comp.preset not in self.opt_output.option
+            )
         ):
             msg = "Compression preset does not match export option\n"
             msg += "You may continue, but the files will need to be compressed again before export\n"
@@ -360,7 +396,7 @@ class Job:
             response = self.executor.cb_return.get_response()
 
             if response is False:
-                return False
+                return False, None
 
         for param, value in (
             ("fps_power", self.opt_comp.fps_power),
@@ -419,7 +455,7 @@ class Job:
             response = self.executor.cb_return.get_response()
 
             if response is False:
-                return False
+                return False, None
 
         # Warn about in/output directories that might contain other files
         # Directory is safe if the name is stickers_input/stickers_output, or
@@ -456,13 +492,13 @@ class Job:
                 response = self.executor.cb_return.get_response()
 
                 if response is False:
-                    return False
+                    return False, None
 
                 break
 
-        return True
+        return True, None
 
-    def cleanup(self) -> bool:
+    def cleanup(self) -> Tuple[bool, None]:
         # If input is 'From local directory', then we should keep files in input/output directory as it maybe edited by user
         # If input is not 'From local directory', then we should move files in input/output directory as new files will be downloaded
         # Output directory should be cleanup unless no_compress is true (meaning files in output directory might be edited by user)
@@ -511,10 +547,10 @@ class Job:
                 new_path = Path(archive_dir, old_path.name)
                 old_path.rename(new_path)
 
-        return True
+        return True, None
 
-    def download(self) -> bool:
-        downloaders: List[Callable[..., bool]] = []
+    def download(self) -> Tuple[bool, str]:
+        downloaders: List[Callable[..., Tuple[int, int]]] = []
 
         if self.opt_input.option == "signal":
             downloaders.append(DownloadSignal.start)
@@ -522,7 +558,7 @@ class Job:
         if self.opt_input.option == "line":
             downloaders.append(DownloadLine.start)
 
-        if self.opt_input.option == "telegram":
+        if self.opt_input.option.startswith("telegram"):
             downloaders.append(DownloadTelegram.start)
 
         if self.opt_input.option == "kakao":
@@ -531,32 +567,44 @@ class Job:
         if self.opt_input.option == "viber":
             downloaders.append(DownloadViber.start)
 
+        if self.opt_input.option.startswith("discord"):
+            downloaders.append(DownloadDiscord.start)
+
         if len(downloaders) > 0:
             self.executor.cb("Downloading...")
         else:
-            self.executor.cb("Nothing to download")
-            return True
+            self.executor.cb("Skipped download (No files to download)")
+            return True, "Download: Skipped (No files to download)"
 
         self.executor.start_workers(processes=1)
 
         for downloader in downloaders:
             self.executor.add_work(
                 work_func=downloader,
-                work_args=(self.opt_input.url, self.opt_input.dir, self.opt_cred),
+                work_args=(self.opt_input, self.opt_cred),
             )
 
         self.executor.join_workers()
 
         # Return False if any of the job returns failure
+        stickers_ok = 0
+        stickers_total = 0
+        success = True
         for result in self.executor.results_list:
-            if result is False:
-                return False
+            stickers_ok += result[0]
+            stickers_total += result[1]
+            success = (
+                success if stickers_ok == stickers_total and stickers_ok > 0 else False
+            )
 
-        return True
+        return (
+            success,
+            f"Download: {stickers_ok}/{stickers_total} stickers success",
+        )
 
-    def compress(self) -> bool:
+    def compress(self) -> Tuple[bool, str]:
         if self.opt_comp.no_compress is True:
-            self.executor.cb("no_compress is set to True, skip compression")
+            self.executor.cb("Skipped compression (no_compress is set to True)")
             in_dir_files = [
                 i
                 for i in sorted(self.opt_input.dir.iterdir())
@@ -583,7 +631,7 @@ class Job:
                     src_f = Path(self.opt_input.dir, i.name)
                     dst_f = Path(self.opt_output.dir, i.name)
                     shutil.copy(src_f, dst_f)
-            return True
+            return True, "Compress: Skipped (no_compress is set to True)"
         msg = "Compressing..."
 
         input_dir = Path(self.opt_input.dir)
@@ -598,15 +646,20 @@ class Job:
 
             if not in_f.is_file():
                 continue
-            if CodecInfo.get_file_ext(i) in (".txt", ".m4a") or Path(i).stem == "cover":
+            if (
+                CodecInfo.get_file_ext(i) in (".txt", ".m4a")
+                or (
+                    self.opt_comp.preset != "signal" and Path(i).stem == "cover"
+                )  # Signal cover has same spec as normal sticker
+            ):
                 shutil.copy(in_f, output_dir / i.name)
             else:
                 in_fs.append(i)
 
         in_fs_count = len(in_fs)
         if in_fs_count == 0:
-            self.executor.cb("No files to compress")
-            return True
+            self.executor.cb("Skipped compression (No files to compress)")
+            return True, "Compress: Skipped (No files to compress)"
 
         self.executor.cb(msg)
         self.executor.cb(
@@ -625,21 +678,35 @@ class Job:
 
         self.executor.join_workers()
 
-        # Return False if any of the job returns failure
+        success = True
+        stickers_ok = 0
+        stickers_total = 0
+        fails: List[str] = []
         for result in self.executor.results_list:
+            stickers_total += 1
             if result[0] is False:
-                return False
+                success = False
+                fails.append(str(result[1]))
+            else:
+                stickers_ok += 1
 
-        return True
+        msg_append = ""
+        if success is False:
+            msg_append = " (Failed: " + ", ".join(fails) + ")"
 
-    def export(self) -> bool:
+        return (
+            success,
+            f"Compress: {stickers_ok}/{stickers_total} stickers success" + msg_append,
+        )
+
+    def export(self) -> Tuple[bool, str]:
         if self.opt_output.option == "local":
-            self.executor.cb("Saving to local directory only, nothing to export")
-            return True
+            self.executor.cb("Skipped export (Saving to local directory only)")
+            return True, "Export: Skipped (Saving to local directory only)"
 
         self.executor.cb("Exporting...")
 
-        exporters: List[Callable[..., List[str]]] = []
+        exporters: List[Callable[..., Tuple[int, int, List[str]]]] = []
 
         if self.opt_output.option == "whatsapp":
             exporters.append(CompressWastickers.start)
@@ -647,15 +714,15 @@ class Job:
         if self.opt_output.option == "signal":
             exporters.append(UploadSignal.start)
 
-        if self.opt_output.option == "telegram":
-            exporters.append(UploadTelegram.start)
-
-        if self.opt_output.option == "telegram_emoji":
+        if self.opt_output.option.startswith("telegram"):
             exporters.append(UploadTelegram.start)
 
         if self.opt_output.option == "imessage":
             exporters.append(XcodeImessage.start)
 
+        if self.opt_output.option == "viber":
+            exporters.append(UploadViber.start)
+
         self.executor.start_workers(processes=1)
 
         for exporter in exporters:
@@ -666,8 +733,12 @@ class Job:
 
         self.executor.join_workers()
 
+        stickers_ok = 0
+        stickers_total = 0
         for result in self.executor.results_list:
-            self.out_urls.extend(result)
+            stickers_ok += result[0]
+            stickers_total += result[1]
+            self.out_urls.extend(result[2])
 
         if self.out_urls:
             with open(
@@ -676,29 +747,6 @@ class Job:
                 f.write("\n".join(self.out_urls))
         else:
             self.executor.cb("An error occured while exporting stickers")
-            return False
+            return False, f"Export: {stickers_ok}/{stickers_total} stickers success"
 
-        return True
-
-    def report(self) -> bool:
-        msg = "##########\n"
-        msg += "Summary:\n"
-        msg += "##########\n"
-        msg += "\n"
-
-        if self.compress_fails:
-            msg += f'Warning: Could not compress the following {len(self.compress_fails)} file{"s" if len(self.compress_fails) > 1 else ""}:\n'
-            msg += "\n".join(self.compress_fails)
-            msg += "\n"
-            msg += "\nConsider adjusting compression parameters"
-            msg += "\n"
-
-        if self.out_urls:
-            msg += "Export results:\n"
-            msg += "\n".join(self.out_urls)
-        else:
-            msg += "Export result: None"
-
-        self.executor.cb(msg)
-
-        return True
+        return True, f"Export: {stickers_ok}/{stickers_total} stickers success"
